/*
 * Copyright (C) 1999-2003 Alex Scott
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*! \file iparser.h
   INI file type parser v0.2
*/

#include <stdio.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#include "cuf-global.h"
#include "llist.h"
#include "mimeproc.h"
#include "strings.h"
#include "win32lacks.h"

#ifndef _IPARSER_H_
# define _IPARSER_H_

# define SAFE_NUM_ALLOC		30
# define SNA			SAFE_NUM_ALLOC

/*!
   \desc
   An Option Type.
*/
typedef enum _IOptType {
	/* Plain variables */
	IBool,				//!< Variable is of type int
	IChar,				//!< Variable is of type char
	IInt,				//!< Variable is of type int
	ILong,				//!< Variable is of type long
	IFloat,				//!< Variable is of type float
	IDouble,			//!< Variable is of type double
	IString,			//!< Variable is of type char *
	IDString,			//!< Variable is of type #DString

	/* Functions used when reading */
	IFuncVoid,			//!< Variable is of type #FuncVoid
	IFuncBool,			//!< Variable is of type #FuncBool
	IFuncChar,			//!< Variable is of type #FuncChar
	IFuncInt,			//!< Variable is of type #FuncInt
	IFuncLong,			//!< Variable is of type #FuncLong
	IFuncFloat,			//!< Variable is of type #FuncFloat
	IFuncDouble,			//!< Variable is of type #FuncDouble
	IFuncString,			//!< Variable is of type #FuncString
	IFuncDString,			//!< Variable is of type #FuncDString

	/* Functions used when writing */
	IFuncWVoid,			//!< Variable is of type #FuncWVoid
	IFuncWBool,			//!< Variable is of type #FuncWBool
	IFuncWChar,			//!< Variable is of type #FuncWChar
	IFuncWInt,			//!< Variable is of type #FuncWInt
	IFuncWLong,			//!< Variable is of type #FuncWLong
	IFuncWFloat,			//!< Variable is of type #FuncWFloat
	IFuncWDouble,			//!< Variable is of type #FuncWDouble
	IFuncWString,			//!< Variable is of type #FuncWString
	IFuncWDString			//!< Variable is of type #FuncWDString
} IOptType;

/*!
   \desc
   Option member, only to be modifyed internaly.
*/
typedef struct _IOption {
	IOptType iot;			// Iparser Option Type
	char *sw;			// Search Word
	void *var;			// Assoated Variable
} IOption;

/*!
   \desc
   Options list, only to be modifyed internaly.
*/
typedef struct _IOpts {
	Llist *list;			// Linked List of Options
	int allowfrees;			// If Allowing Internal Frees.
} IOpts;

/*** Iparser Public Functions ***/

IOpts *ioption_init (void);
void ioption_add (IOpts *opts, char *sw, IOptType iot, void *var);
void ioption_allowfrees_set (IOpts *opts, int allowfrees);
int ioption_parse (IOpts *opts, char *filename);
int ioption_write (IOpts *opts, char *filename);
void ioption_done (IOpts *opts);

#endif /* _IPARSER_H_ */
