/*
 * Copyright (C) 1999-2003 Alex Scott
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*! \file popts.h
   Command line options parser, v0.3
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "cuf-global.h"
#include "array.h"
#include "llist.h"

#ifndef _POPTS_H_
# define _POPTS_H_

#define DESC_WINDOW_WIDTH		78
#define DESC_COLUMN_START		30

typedef void (*FuncString2d) (char **);

/*!
   \desc
   The data type.
*/
typedef enum _OType {
	// Regular variables
	OBool,				//!< int
	OChar,				//!< char
	OInt,				//!< int
	OLong,				//!< long
	OFloat,				//!< float
	ODouble,			//!< double
	OString,			//!< char *

	// Functions
	OFuncVoid,			//!< #FuncVoid
	OFuncBool,			//!< #FuncBool
	OFuncChar,			//!< #FuncChar
	OFuncInt,			//!< #FuncInt
	OFuncLong,			//!< #FuncLong
	OFuncFloat,			//!< #FuncFloat
	OFuncDouble,			//!< #FuncDouble
	OFuncString,			//!< #FuncString

	// Specialty
	OString2d,			//!< char **
	OFuncString2d,			//!< void func (char **)
	OProgramname			//!< char *
} OType;

/*
   NOTE: Weather you add OProgramname or OString2d or OFuncString2d to the
     letter or word lists they will always be added to the word list. I
     suggest just adding to the word list to skip any extra if statements
     that might be nessary.
*/

/*!
   \desc
   Option description, only to be modifyed internaly.
*/
typedef struct _COption_description {
	unsigned int letterid;		// Letter ID for description
	unsigned int wordid;		// Word ID for description
	char *desc;			// The description of the option
} COption_description;

/*!
   \desc
   Option letter, only to be modifyed internaly.
*/
typedef struct _COption_letter {
	unsigned int id;		// Letter option ID
	OType ot;			// Option Type
	char sl;			// Search Leter
	void *var;			// Variable
} COption_letter;

/*!
   \desc
   Option word, only to be modifyed internaly.
*/
typedef struct _COption_word {
	unsigned int id;		// Word Option ID
	OType ot;			// Option Type
	char *sw;			// Search Word
	void *var;			// Variable
} COption_word;


/*** Public Functions ***/

void popts_init (void);
void popts_description_add (unsigned int letterid, unsigned int wordid,
                            char *desc);
unsigned int popts_letter_add (char sl, OType ot, void *var);
unsigned int popts_word_add (char *sw, OType ot, void *var);
void popts_help_add (void);
void popts_descriptions_print (void);
int popts_parse (int argc, char **argv);
void popts_done (void);

#endif /* _POPTS_H_ */
