/*
 * Copyright (C) 1999-2003 Alex Scott
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*! \file popts2.h
   Command line options parser, v0.3
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "cuf-global.h"
#include "array.h"
#include "llist.h"

#ifndef _POPTS2_H_
# define _POPTS2_H_

# define DESC_WINDOW_WIDTH		78
# define DESC_COLUMN_START		34

typedef void (*FuncString2d) (char **);

/*!
   \desc
   The data type.
*/
typedef enum _OType {
	// Regular variables
	OBool,				//!< int, The bool inverts the current setting. Multiable calls to the bool on the command line will invert with each time the switch is parsed.
	OChar,				//!< char
	OInt,				//!< int
	OLong,				//!< long
	OFloat,				//!< float
	ODouble,			//!< double
	OString,			//!< char *, If the pointer is NULL, then memory will be allocated. If the pointer is not null it will be freed and new memory will be allocated.

	// Functions
	OFuncVoid,			//!< #FuncVoid, Just the function is called with no arguments.
	OFuncBool,			//!< #FuncBool, The function is called with TRUE as the argument.
	OFuncChar,			//!< #FuncChar
	OFuncInt,			//!< #FuncInt
	OFuncLong,			//!< #FuncLong
	OFuncFloat,			//!< #FuncFloat
	OFuncDouble,			//!< #FuncDouble
	OFuncString,			//!< #FuncString

	// Specialty
	OString2d,			//!< char **
	OFuncString2d,			//!< void func (char **)
	OProgramname			//!< char *
} OType;

/*!
   \desc
   Option Class.
*/
typedef enum _OClass {
	OCLetter,			//!< Single Character switch
	OCWord,				//!< String switch
	OCSpecial			//!< Special (not switched and program name).
} OClass;

/*!
   \desc
   Description Member, (Internal Only).
*/
typedef struct _DescMember {
	unsigned int did;		// This descriptions id.
	unsigned int num_ids;		// Number of id's assoicated with this option.
	unsigned int *ids;		// The array of id's
	char *desc;			// The description
} DescMember;

/*!
   \desc
   Description Catagory, (Internal Only).
*/
typedef struct _DescCatagory {
	char *cname;			// The name of the catagory
	char *text;			// Any extra text about the catagory
	Llist *members;			// a linked list of description memebers
} DescCatagory;

/*!
   \desc
   Command Option, (Internal Only).
*/
typedef struct _COption {
	unsigned int id;		// Id
	int printed;			// Internal, if it was printed or not.
	OClass oc;			// Option Class
	OType ot;			// Option Type
	char *s;			// search word/letter
	void *var;			// Manipulation variable
} COption;

unsigned int popts2_add (char *s, OClass oc, OType ot, void *var);
void popts2_desc_cat_add (char *cname, char *text);
int popts2_desc_add (char *cname, char *desc);
void popts2_desc_connect (unsigned int did, unsigned int oid);
void popts2_desc_print (void);
void popts2_help_add (void);
void popts2_parse (int argc, char **argv);
void popts2_done (void);

#endif /* _POPTS2_H_ */
