/*
 * Copyright (C) 1999-2003 Alex Scott
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Emulations of functions missing from the WIN32 platform */

#ifdef WIN32  // no need to use this if your not on a win32 platform

# include <stdio.h>
# include <errno.h>

# include <windows.h>
# include <direct.h>
# include <io.h>

# include "cuf-global.h"

# ifndef _WIN32LACKS_H_
#  define _WIN32LACKS_H_

// emulation of the stat macros
#  define S_ISDIR(m) ((m) & S_IFDIR)
// More to be added when i need them or their requested

#  ifndef strcasecmp
#   define strcasecmp win32_strcasecmp
#  endif
#  ifndef strncasecmp
#   define strncasecmp win32_strncasecmp
#  endif
#  ifndef snprintf
#   define snprintf win32_snprintf
#  endif

/* WIN32 emulations of POSIX/BSD dir functions */
#  ifndef opendir
#   define opendir win32_opendir
#  endif
#  ifndef readdir
#   define readdir win32_readdir
#  endif
#  ifndef rewinddir
#   define rewinddir win32_rewinddir
#  endif
#  ifndef closedir
#   define closedir win32_closedir
#  endif

#  define NAME_MAX 255

// Use to keep the main function the same as were use to in unix.
#  include <stdlib.h> /* needed for __argc and __argv */
#  define WIN32MAIN \
struct HINSTANCE__; \
int _stdcall WinMain (struct HINSTANCE__ *hInstance, \
                      struct HINSTANCE__ *hPrevInstance, \
                      char *lpszCmdLine, int nCmdShow) { \
    return main (__argc, __argv); \
}


struct dirent {
	long d_ino;			// not set in my WIN32 wrapper
	off_t d_off;			// not set in my WIN32 wrapper
	unsigned short d_reclen;
	char d_name[NAME_MAX + 1];
};

typedef struct _DIR {
	char *dir_name;
	int just_opened;
	HANDLE find_file_handle;
	void *find_file_data;
	struct dirent readdir_result;
} DIR;

typedef unsigned int mode_t;

char *win32_basename (char *file);

int win32_strcasecmp (const char *sa, const char *sb);
int win32_strncasecmp (const char *sa, const char *sb, size_t n);
int win32_snprintf (char *str, size_t n, const char *format, ...);

DIR *win32_opendir (const char *dirname);
struct dirent *win32_readdir (DIR *dir);
void win32_rewinddir (DIR *dir);
int win32_closedir (DIR *dir);

# endif /* _WIN32LACKS_H_ */

#else

// If not win32, it dosent do anything.
# define WIN32MAIN

#endif /* WIN32 */

